/**
 * Account helper
 *
 * @author Pierre Hubert
 */

import {
	ArrayBufferToBase64,
	base64NoPaddingToUint8Array,
} from "../utils/Base64Utils";
import { serverRequest } from "./APIHelper";

export interface AuthKey {
	name: string;
	id: number;
}

export interface AuthOptions {
	reset_token: string;
	keys: AuthKey[];
}

export interface AdminAccount {
	id: number;
	name: string;
	email: string;
	time_create: number;
}

export interface NewAdminGeneralSettings {
	id: number;
	name: string;
	email: string;
}

export interface AdminAccountKey {
	id: number;
	name: string;
	time_add: number;
}

export interface AdminResetToken {
	token: string;
	expire: number;
}

const SESSION_STORAGE_TOKEN = "auth_token";

let currentAccount: AdminAccount | null = null;

export class AccountHelper {
	/**
	 * Check whether access token are available
	 * or not
	 */
	static get hasAccessToken(): boolean {
		return sessionStorage.getItem(SESSION_STORAGE_TOKEN) != null;
	}

	/**
	 * Get current user access token
	 */
	static get accessToken(): string {
		const token = sessionStorage.getItem(SESSION_STORAGE_TOKEN);
		if (!token) throw new Error("No access token for now!");
		return token;
	}

	/**
	 * Get authentication options for a given email address
	 *
	 * @param mail Requested email
	 */
	static async getAuthOptions(mail: string): Promise<AuthOptions> {
		return await serverRequest("accounts/auth_options", {
			mail: mail,
		});
	}

	/**
	 * Attempt to authenticate user using a reset token
	 *
	 * @param mail Email address of the admin
	 * @param token The reset token
	 */
	static async authWithResetToken(mail: string, token: string) {
		const res = await serverRequest("accounts/auth_with_reset_token", {
			mail: mail,
			token: token,
		});

		sessionStorage.setItem(SESSION_STORAGE_TOKEN, res.token);
	}

	/**
	 * Get information about an administrator
	 *
	 * @param id The ID of the target administrator
	 */
	static async getAdminInfo(id: number): Promise<AdminAccount> {
		return await serverRequest("accounts/info", {
			id: id,
		});
	}

	/**
	 * Attempt to refresh current account information
	 */
	static async refreshCurrentAccountInfo() {
		const res = await serverRequest("accounts/info", {}, true);

		if (res.error) {
			if (res.error > 0) {
				sessionStorage.removeItem(SESSION_STORAGE_TOKEN);
				document.location.href = document.location.href + "";
			}
			throw new Error("Request failed!");
		}

		currentAccount = res;
	}

	/**
	 * Get current account information
	 */
	static get currentAccount(): AdminAccount {
		if (currentAccount == null) throw new Error("Current account is null!");
		return currentAccount;
	}

	/**
	 * Sign out current user
	 */
	static async signOut() {
		try {
			await serverRequest("accounts/sign_out");
		} catch (e) {
			console.error(e);
		}

		sessionStorage.removeItem(SESSION_STORAGE_TOKEN);
		document.location.href = document.location.href + "";
	}

	/**
	 * Update an admin's settings
	 *
	 * @param a New settings
	 */
	static async UpdateGeneralSettings(s: NewAdminGeneralSettings) {
		await serverRequest("accounts/update_general_settings", {
			id: s.id,
			name: s.name,
			email: s.email,
		});
	}

	/**
	 * Generate a access reset token for an admin
	 *
	 * @param adminID The id of the target administrator
	 */
	static async GenerateResetToken(adminID: number): Promise<AdminResetToken> {
		return await serverRequest("accounts/generate_reset_token", {
			id: adminID,
		});
	}

	/**
	 * First step of access key enrollment
	 */
	static async GetKeyRegistrationChallenge(): Promise<any> {
		const res = await serverRequest("accounts/challenge_register_key");
		res.publicKey.challenge = base64NoPaddingToUint8Array(
			res.publicKey.challenge
		);
		res.publicKey.user.id = base64NoPaddingToUint8Array(
			res.publicKey.user.id
		);

		return res;
	}

	/**
	 * Register key
	 *
	 * @param name The name of the key to create
	 * @param cred The credentials to register
	 */
	static async RegisterKey(name: string, cred: any): Promise<void> {
		const res = {
			id: cred.id,
			rawId: ArrayBufferToBase64(cred.rawId),
			type: cred.type,
			response: {
				attestationObject: ArrayBufferToBase64(
					cred.response.attestationObject
				),
				clientDataJSON: ArrayBufferToBase64(
					cred.response.clientDataJSON
				),
			},
		};

		await serverRequest("accounts/register_key", {
			name: name,
			key: JSON.stringify(res),
		});
	}

	/**
	 * First step of security key authentication
	 *
	 * @param mail Target admin account email address
	 * @param key The key to use to authentifcate
	 */
	static async GetAuthenticationChallenge(
		mail: string,
		key: AuthKey
	): Promise<any> {
		const res = await serverRequest("accounts/challenge_auth_with_key", {
			mail: mail,
			key_id: key.id,
		});

		res.publicKey.challenge = base64NoPaddingToUint8Array(
			res.publicKey.challenge
		);

		for (let cred of res.publicKey.allowCredentials) {
			cred.id = base64NoPaddingToUint8Array(cred.id);
		}

		return res;
	}

	/**
	 * Attempt to sign in using security key
	 *
	 * @param mail Target admin account email address
	 * @param key Key used to authenticate
	 * @param cred Response to authentication
	 */
	static async AuthenticateWithKey(
		mail: string,
		key: AuthKey,
		cred: any
	): Promise<any> {
		const creds = {
			id: cred.id,
			rawId: ArrayBufferToBase64(cred.rawId),
			type: cred.type,
			response: {
				authenticatorData: ArrayBufferToBase64(
					cred.response.authenticatorData
				),
				clientDataJSON: ArrayBufferToBase64(
					cred.response.clientDataJSON
				),
				signature: ArrayBufferToBase64(cred.response.signature),
				userHandle: cred.response.userHandle,
			},
		};

		const res = await serverRequest("accounts/auth_with_key", {
			mail: mail,
			key_id: key.id,
			credential: JSON.stringify(creds),
		});

		sessionStorage.setItem(SESSION_STORAGE_TOKEN, res.token);
	}

	/**
	 * Get the list of keys of an admin
	 *
	 * @param adminID The id of the target administrator
	 */
	static async GetAdminKeys(adminID: number): Promise<AdminAccountKey[]> {
		return await serverRequest("accounts/keys", {
			id: adminID,
		});
	}

	/**
	 * Delete an admin auth key
	 *
	 * @param adminID The id of the target admin
	 * @param keyID The id of the key to delete
	 */
	static async DeleteAuthKey(adminID: number, keyID: number) {
		return await serverRequest("accounts/delete_auth_key", {
			adminID: adminID,
			keyID: keyID,
		});
	}
}
