/**
 * Account settings route
 *
 * @author Pierre Hubert
 */

import {
	Button,
	Divider,
	Grid,
	IconButton,
	Paper,
	Table,
	TableBody,
	TableCell,
	TableHead,
	TableRow,
	TextField,
	Typography,
} from "@material-ui/core";
import { Delete } from "@material-ui/icons";
import React from "react";
import { useParams } from "react-router-dom";
import {
	AccountHelper,
	AdminAccount,
	AdminAccountKey,
} from "../../helpers/AccountHelper";
import { CopyToClipboard } from "../../utils/ClipboardUtils";
import { AsyncWidget } from "../widgets/AsyncWidget";
import {
	input,
	matAlert,
	matConfirm,
	snackbar,
} from "../widgets/DialogsProvider";
import { PageTitle } from "../widgets/PageTitle";
import { TimestampWidget } from "../widgets/TimestampWidget";

export function AccountSettingsRoute() {
	let params: any = useParams();

	return <AccountSettingsRouteInner id={Number(params.id)} />;
}

interface SettingsRouteProps {
	id: number;
}

interface SettingsRouteState {
	account: AdminAccount;
}

class AccountSettingsRouteInner extends React.Component<
	SettingsRouteProps,
	SettingsRouteState
> {
	constructor(props: any) {
		super(props);

		this.load = this.load.bind(this);
		this.build = this.build.bind(this);
	}

	async load() {
		const account = await AccountHelper.getAdminInfo(this.props.id);

		this.setState({ account: account });
	}

	render() {
		return (
			<AsyncWidget
				key={this.props.id}
				errorMessage="Failed to load admin information!"
				onBuild={this.build}
				load={this.load}
			></AsyncWidget>
		);
	}

	build() {
		return (
			<div>
				<PageTitle name="Account settings"></PageTitle>
				<Grid container spacing={2}>
					<GeneralSettings
						admin={this.state.account}
					></GeneralSettings>

					<KeySettingsSection
						admin={this.state.account}
					></KeySettingsSection>
				</Grid>
			</div>
		);
	}
}

class GeneralSettings extends React.Component<
	{ admin: AdminAccount },
	{ newName: string; newEmail: string }
> {
	constructor(p: any) {
		super(p);

		this.state = {
			newName: this.props.admin.name,
			newEmail: this.props.admin.email,
		};

		this.changedName = this.changedName.bind(this);
		this.changedEmail = this.changedEmail.bind(this);
		this.handleSubmit = this.handleSubmit.bind(this);
	}

	changedName(e: React.ChangeEvent<HTMLInputElement>) {
		this.setState({ newName: e.target.value });
	}

	changedEmail(e: React.ChangeEvent<HTMLInputElement>) {
		this.setState({ newEmail: e.target.value });
	}

	get isValid(): boolean {
		return this.state.newEmail.length > 2 && this.state.newName.length > 2;
	}

	async handleSubmit() {
		try {
			if (!this.isValid) return;

			await AccountHelper.UpdateGeneralSettings({
				id: this.props.admin.id,
				name: this.state.newName,
				email: this.state.newEmail,
			});

			snackbar("Successfully updated admin settings!");
		} catch (e) {
			console.error(e);
			matAlert("Failed to update admin settings!");
		}
	}

	render() {
		return (
			<SettingsSection title="General settings">
				<div style={{ margin: "10px" }}>
					<TextField
						required
						label="Name"
						value={this.state.newName}
						onChange={this.changedName}
						style={{ width: "100%", paddingBottom: "20px" }}
					/>

					<TextField
						required
						label="Email"
						value={this.state.newEmail}
						onChange={this.changedEmail}
						type="mail"
						style={{ width: "100%", paddingBottom: "20px" }}
					/>

					<div style={{ textAlign: "right" }}>
						<Button
							style={{ alignSelf: "end", marginRight: "10px" }}
							disabled={!this.isValid}
							onClick={this.handleSubmit}
						>
							Update
						</Button>
					</div>
				</div>
			</SettingsSection>
		);
	}
}

export class KeySettingsSection extends React.Component<
	{ admin: AdminAccount },
	{ keys: AdminAccountKey[]; counter: number }
> {
	constructor(props: any) {
		super(props);

		this.state = {
			keys: [],
			counter: 1,
		};

		this.load = this.load.bind(this);
		this.build = this.build.bind(this);
		this.generateResetToken = this.generateResetToken.bind(this);
		this.registerNewKey = this.registerNewKey.bind(this);
		this.deleteKey = this.deleteKey.bind(this);
	}

	async load() {
		const keys = await AccountHelper.GetAdminKeys(this.props.admin.id);

		this.setState({ keys: keys });
	}

	async generateResetToken() {
		try {
			if (
				!(await matConfirm(
					"Do you really want to generate a reset token for this account?"
				))
			)
				return;

			const token = await AccountHelper.GenerateResetToken(
				this.props.admin.id
			);

			CopyToClipboard(token.token);
			snackbar("Reset token was successfully copied to the clipboard!");
		} catch (e) {
			console.error(e);
			matAlert("Failed to generate a token!");
		}
	}

	async registerNewKey() {
		try {
			const challenge = await AccountHelper.GetKeyRegistrationChallenge();
			const credential = await navigator.credentials.create(challenge);

			if (credential == null) throw new Error("Operation aborted!");

			const name = await input({
				label: "Key name",
				maxLength: 40,
				minLength: 2,
			});

			await AccountHelper.RegisterKey(name, credential);

			snackbar("Successfully enrolled a new key!");

			this.setState({ counter: this.state.counter + 1 });
		} catch (e) {
			console.error(e);
			matAlert("Failed to register a new key!");
		}
	}

	async deleteKey(key: AdminAccountKey) {
		try {
			if (
				!(await matConfirm(
					"Do you really want to delete the key '" + key.name + "' ?"
				))
			)
				return;

			await AccountHelper.DeleteAuthKey(this.props.admin.id, key.id);

			snackbar("The key was successfully deleted!");
			this.setState({ counter: this.state.counter + 1 });
		} catch (e) {
			console.error(e);
			matAlert("Failed to delete key!");
		}
	}

	render() {
		return (
			<AsyncWidget
				errorMessage="Failed to load admin keys!"
				load={this.load}
				onBuild={this.build}
				key={this.props.admin.id + "-" + this.state.counter}
			></AsyncWidget>
		);
	}

	build() {
		return (
			<SettingsSection title="Security keys">
				<Table aria-label="simple table">
					<TableHead>
						<TableRow>
							<TableCell>Key name</TableCell>
							<TableCell align="right">Date added</TableCell>
							<TableCell align="right"></TableCell>
						</TableRow>
					</TableHead>
					<TableBody>
						{this.state.keys.map((key) => (
							<TableRow key={key.id}>
								<TableCell component="th" scope="row">
									{key.name}
								</TableCell>
								<TableCell align="right">
									<TimestampWidget time={key.time_add} />
								</TableCell>
								<TableCell align="right">
									<IconButton
										aria-label="delete"
										size="small"
										onClick={() => this.deleteKey(key)}
									>
										<Delete />
									</IconButton>
								</TableCell>
							</TableRow>
						))}
					</TableBody>
				</Table>

				<Divider />

				{/* Action buttons */}
				<div
					style={{
						textAlign: "right",
						margin: "5px 10px",
					}}
				>
					<Button onClick={this.generateResetToken}>
						New reset token
					</Button>
					<Button
						disabled={
							this.props.admin.id !==
							AccountHelper.currentAccount.id
						}
						onClick={this.registerNewKey}
					>
						Register a new key
					</Button>
				</div>
			</SettingsSection>
		);
	}
}

function SettingsSection(p: { title: string; children?: React.ReactNode }) {
	return (
		<Grid item sm={6}>
			<Paper>
				<Typography variant="h6" style={{ padding: "10px 15px " }}>
					{p.title}
				</Typography>
				<Divider />
				<div
					style={{
						display: "flex",
						flexDirection: "column",
					}}
				>
					{p.children}
				</div>
			</Paper>
		</Grid>
	);
}
